@@ -196,21 +196,17 @@ macro_rules! table {
 ///
 /// # Examples
 /// ```
-/// use janetrs::{janet_mod, lowlevel, Janet};
+/// use janetrs::{janet_mod, Janet, janet_fn};
 ///
-/// #[no_mangle]
-/// unsafe extern "C" fn rust_hello(argc: i32, _argv: *mut lowlevel::Janet) -> lowlevel::Janet {
-///     lowlevel::janet_fixarity(argc, 0);
+/// #[janet_fn(arity(fix(0)))]
+/// fn rust_hello(args: &mut [Janet]) -> Janet {
 ///     println!("Hello from Rust!");
-///
-///     Janet::nil().into()
+///     Janet::nil()
 /// }
 ///
-/// #[no_mangle]
-/// unsafe extern "C" fn hi(argc: i32, _argv: *mut lowlevel::Janet) -> lowlevel::Janet {
-///     lowlevel::janet_fixarity(argc, 0);
-///
-///     Janet::from("Hi! My name is GrayJack!").into()
+/// #[janet_fn(arity(fix(0)))]
+/// fn hi(args: &mut [Janet]) -> Janet {
+///     Janet::from("Hi! My name is GrayJack!")
 /// }
 ///
 /// janet_mod!("rust";
@@ -218,6 +214,7 @@ macro_rules! table {
 ///     {"hi", hi, "(rust/hi)\n\nHi! My name is..."}
 /// );
 /// ```
+#[deprecated(since = "0.4.0", note = "use `declare_janet_mod` instead")]
 #[macro_export]
 macro_rules! janet_mod {
     ($mod_name:literal; $({$fn_name:literal, $fn:expr, $fn_doc:literal}),* $(,)?) => {
@@ -233,12 +230,12 @@ macro_rules! janet_mod {
 
         #[no_mangle]
         pub unsafe extern "C" fn _janet_init(env: *mut $crate::lowlevel::JanetTable) {
-            $crate::lowlevel::janet_cfuns(env, concat!($mod_name, "\0").as_ptr() as *const i8, [
+            $crate::lowlevel::janet_cfuns(env, concat!($mod_name, "\0").as_ptr() as *const _, [
                 $(
                     $crate::lowlevel::JanetReg {
-                        name: concat!($fn_name, "\0").as_ptr() as *const i8,
+                        name: concat!($fn_name, "\0").as_ptr() as *const _,
                         cfun: Some($fn),
-                        documentation: concat!($fn_doc, "\0").as_ptr() as *const i8,
+                        documentation: concat!($fn_doc, "\0").as_ptr() as *const _,
                     },
                 )*
 
@@ -252,9 +249,9 @@ macro_rules! janet_mod {
     };
 }
 
-/// Causes a panic in the Janet side. Diferently of the Rust `panic!` macro, this macro
-/// does **not** terminate the current thread. Instead, it sends a error signal with the
-/// passed message where the Janet runtime takes care to properly exit.
+/// Causes a panic in the Janet side (exception). Diferently of the Rust `panic!` macro,
+/// this macro does **not** terminate the current thread. Instead, it sends a error signal
+/// with the passed message where the Janet runtime takes care to properly exit.
 ///
 /// # Examples
 /// ```ignore
@@ -269,19 +266,19 @@ macro_rules! janet_mod {
 #[macro_export]
 macro_rules! jpanic {
     () => {
-        $crate::util::_panic($crate::Janet::from("explicity panic"));
+        $crate::util::_panic($crate::Janet::from("explicity panic"))
     };
     ($msg:expr $(,)?) => {
-        $crate::util::_panic($crate::Janet::from($msg));
+        $crate::util::_panic($crate::Janet::from($msg))
     };
     ($msg:expr, $($arg:tt)+) => {
-        $crate::util::_panic($crate::Janet::from(::alloc::format!($msg, $($arg)+).as_str()));
+        $crate::util::_panic($crate::Janet::from(::alloc::format!($msg, $($arg)+).as_str()))
     };
 }
 
-/// Causes a panic in the Janet side. Diferently of the Rust `panic!` macro, this macro
-/// does **not** terminate the current thread. Instead, it sends a error signal with the
-/// passed message where the Janet runtime takes care to properly exit.
+/// Causes a panic in the Janet side (exception). Diferently of the Rust `panic!` macro,
+/// this macro does **not** terminate the current thread. Instead, it sends a error signal
+/// with the passed message where the Janet runtime takes care to properly exit.
 ///
 /// # Examples
 /// ```ignore
@@ -296,18 +293,18 @@ macro_rules! jpanic {
 #[macro_export]
 macro_rules! jpanic {
     () => {
-        $crate::util::_panic($crate::Janet::from("explicity panic"));
+        $crate::util::_panic($crate::Janet::from("explicity panic"))
     };
     ($msg:expr $(,)?) => {
-        $crate::util::_panic($crate::Janet::from($msg));
+        $crate::util::_panic($crate::Janet::from($msg))
     };
     ($msg:expr, $($arg:tt)+) => {
-        $crate::util::_panic($crate::Janet::from(::std::format!($msg, $($arg)+).as_str()));
+        $crate::util::_panic($crate::Janet::from(::std::format!($msg, $($arg)+).as_str()))
     };
 }
 
-/// Execute a [`JanetCFunction`] and catch any janet panic that may happen as a
-/// [`Result`].
+/// Execute a function ([`JanetCFunction`], Rust function or extern C function) and catch
+/// any janet panic that may happen as a [`Result`].
 ///
 /// # Examples
 /// ```ignore
@@ -347,7 +344,7 @@ macro_rules! jcatch {
     }};
 }
 
-/// Macro that tries to run a expression, and if it panics in Rust side, it tries to
+/// Macro that tries to run a expression, and if it panics in the Rust side, it tries to
 /// recover from that and pass the Rust panic string to a Janet Panic.
 ///
 /// This uses the [`catch_unwind`] function, and therefore have the same guarantees as it.
@@ -365,6 +362,7 @@ macro_rules! jcatch {
 ///
 /// [`catch_unwind`]: https://doc.rust-lang.org/std/panic/fn.catch_unwind.html
 #[cfg(feature = "std")]
+#[cfg_attr(feature = "_doc", doc(cfg(feature = "std")))]
 #[macro_export]
 macro_rules! jtry {
     ($e:expr) => {{
