@@ -23,6 +23,7 @@ pub mod array;
 pub mod buffer;
 pub mod fiber;
 pub mod function;
+pub mod io;
 #[path = "types/abstract.rs"]
 pub mod janet_abstract;
 pub mod pointer;
@@ -32,12 +33,15 @@ pub mod symbol;
 pub mod table;
 pub mod tuple;
 
+mod random;
+
 pub use array::JanetArray;
 pub use buffer::JanetBuffer;
 pub use fiber::JanetFiber;
 pub use function::{JanetCFunction, JanetFunction};
 pub use janet_abstract::{IsJanetAbstract, JanetAbstract};
 pub use pointer::JanetPointer;
+pub use random::JanetRng;
 pub use string::JanetString;
 pub use structs::JanetStruct;
 pub use symbol::{JanetKeyword, JanetSymbol};
@@ -67,6 +71,7 @@ pub trait DeepEq<Rhs = Self> {
 pub struct JanetConversionError;
 
 #[cfg(feature = "std")]
+#[cfg_attr(feature = "_doc", doc(cfg(feature = "std")))]
 impl error::Error for JanetConversionError {}
 
 impl Display for JanetConversionError {
@@ -314,28 +319,30 @@ impl Janet {
 
     /// Get a dynamic [keywrord](self::JanetKeyword) binding from the environment if it
     /// exists.
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn dynamic(key: impl AsRef<[u8]>) -> Option<Self> {
-        let mut key: JanetBuffer = key.as_ref().into();
-        key.push('\0');
-        // let mut key = String::from(key.to_str().ok()?);
-        // key.push('\0');
+        Self::_dynamic(key.as_ref())
+    }
 
-        let janet = Janet::from(unsafe { evil_janet::janet_dyn(key.as_ptr() as *const _) });
+    fn _dynamic(key: &[u8]) -> Option<Self> {
+        let mut key: JanetBuffer = key.into();
+        key.push('\0');
+
+        let janet = Self::from(unsafe { evil_janet::janet_dyn(key.as_ptr() as *const _) });
 
         if janet.is_nil() { None } else { Some(janet) }
     }
 
     /// Resolve a `symbol` in the core environment.
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn from_core<'a>(symbol: impl Into<JanetSymbol<'a>>) -> Option<Self> {
         let env = JanetEnvironment::default();
         env.resolve(symbol)
     }
 
     /// Wraps a value into a [`Janet`].
-    #[inline]
-    pub fn wrap(value: impl Into<Janet>) -> Self {
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn wrap(value: impl Into<Self>) -> Self {
         value.into()
     }
 
@@ -345,9 +352,88 @@ impl Janet {
         self.into()
     }
 
+    /// Returns the contained value or a provided default
+    ///
+    /// Consumes the `self` argument then, if the conversion to type `T` goes correctly,
+    /// returns the contained value, otherwise if the conversion fails, returns the
+    /// given `default` value for that type.
+    ///
+    /// Arguments passed to `unwrap_or` are eagerly evaluated; if you are passing
+    /// the result of a function call, it is recommended to use
+    /// [`unwrap_or_else`](#unwrap_os_else), which is lazily evaluated.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use janetrs::Janet;
+    /// # let _client = janetrs::client::JanetClient::init().unwrap();
+    ///
+    /// let default = 2.0;
+    /// let x = Janet::number(9.0);
+    /// assert_eq!(x.unwrap_or(default), 9.0);
+    ///
+    /// let x = Janet::boolean(true);
+    /// assert_eq!(x.unwrap_or(default), default);
+    /// ```
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn unwrap_or<T: TryFrom<Self>>(self, default: T) -> T {
+        T::try_from(self).unwrap_or(default)
+    }
+
+    /// Returns the contained value or computes it from a closure.
+    ///
+    /// The closure holds the value of the conversion error to better handle it to decide
+    /// a default value to return.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use janetrs::Janet;
+    /// # let _client = janetrs::client::JanetClient::init().unwrap();
+    ///
+    /// let default = |_| 1.0;
+    /// let x = Janet::number(9.0);
+    /// assert_eq!(x.unwrap_or_else(default), 9.0);
+    ///
+    /// let x = Janet::boolean(true);
+    /// assert_eq!(x.unwrap_or_else(default), 1.0);
+    /// ```
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn unwrap_or_else<T, F>(self, op: F) -> T
+    where
+        T: TryFrom<Self>,
+        F: FnOnce(T::Error) -> T,
+    {
+        T::try_from(self).unwrap_or_else(op)
+    }
+
+    /// Returns the contained value or a default
+    ///
+    /// Consumes the `self` argument then, if the conversion to type `T` goes correctly,
+    /// returns the contained value, otherwise if the conversion fails, returns the
+    /// default value for that type.
+    ///
+    /// # Examples
+    ///
+    /// /// ```
+    /// use janetrs::Janet;
+    /// # let _client = janetrs::client::JanetClient::init().unwrap();
+    ///
+    /// let x = Janet::number(9.0);
+    /// assert_eq!(x.unwrap_or_default(default), 9.0);
+    ///
+    /// let x = Janet::boolean(true);
+    /// assert_eq!(x.unwrap_or_default(default), 0.0);
+    /// ```
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn unwrap_or_default<T>(self) -> T
+    where T: TryFrom<Self> + Default {
+        T::try_from(self).unwrap_or_default()
+    }
+
     /// Tries to unwrap the [`Janet`] into a concrete type that implements
     /// [`TryFrom`]<[`Janet`]>.
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn try_unwrap<T: TryFrom<Self>>(self) -> Result<T, T::Error> {
         T::try_from(self)
     }
@@ -377,7 +463,7 @@ impl Janet {
             TaggedJanet::Tuple(x) => Some(x.len()),
             TaggedJanet::Abstract(_) => {
                 self.get_method("length")
-                    .and_then(|method: Janet| match method.unwrap() {
+                    .and_then(|method: Self| match method.unwrap() {
                         // I think Abstract methods can only be a C function because as far a
                         // I(GrayJack) know, a JanetFunction cannot be created (as in written) by
                         // the public Janet C API.
@@ -385,11 +471,11 @@ impl Janet {
                         _ => None,
                     })
                     .map(|f| {
-                        // Safety: We are trusting that am Abstract Janet method through a C
-                        // function will not cause UB. It can janet panic
+                        // SAFETY: We are trusting that am Abstract Janet method through a C
+                        // function will not cause UB. It can janet panic.
                         unsafe { f(1, [self.inner].as_mut_ptr()) }.into()
                     })
-                    .and_then(|len: Janet| match len.unwrap() {
+                    .and_then(|len: Self| match len.unwrap() {
                         TaggedJanet::Number(x) => {
                             if x >= i32::MIN as f64
                                 && x <= i32::MAX as f64
@@ -425,9 +511,9 @@ impl Janet {
 
     /// Retuns a `Janet` value containing the requested method if it exists.
     #[inline]
-    pub fn get_method(&self, method_name: &str) -> Option<Janet> {
-        let method_name: Janet = JanetKeyword::from(method_name).into();
-        let method: Janet = unsafe { evil_janet::janet_get(self.inner, method_name.inner) }.into();
+    pub fn get_method(&self, method_name: &str) -> Option<Self> {
+        let method_name: Self = JanetKeyword::from(method_name).into();
+        let method: Self = unsafe { evil_janet::janet_get(self.inner, method_name.inner) }.into();
 
         if method.is_nil() {
             return None;
@@ -471,9 +557,9 @@ macro_rules! struct_table_display {
                 let key = unsafe { keys.get_unchecked(i) };
                 let value = unsafe { $id.get_unchecked(key) };
 
-                fmt::Display::fmt(key, $f)?;
+                Display::fmt(key, $f)?;
                 $f.write_str(" : ")?;
-                fmt::Display::fmt(value, $f)?;
+                Display::fmt(value, $f)?;
                 $f.write_str(", ")?;
             }
 
@@ -484,9 +570,9 @@ macro_rules! struct_table_display {
                 let key = unsafe { keys.get_unchecked(i) };
                 let value = unsafe { $id.get_unchecked(key) };
 
-                fmt::Display::fmt(key, $f)?;
+                Display::fmt(key, $f)?;
                 $f.write_str(" : ")?;
-                fmt::Display::fmt(value, $f)?;
+                Display::fmt(value, $f)?;
                 $f.write_str(", ")?;
             }
 
@@ -494,15 +580,15 @@ macro_rules! struct_table_display {
             let key = unsafe { keys.get_unchecked(len - 1) };
             let value = unsafe { $id.get_unchecked(key) };
 
-            fmt::Display::fmt(key, $f)?;
+            Display::fmt(key, $f)?;
             $f.write_str(" : ")?;
-            fmt::Display::fmt(value, $f)?;
+            Display::fmt(value, $f)?;
         } else {
             let mut count = $id.len();
-            for (key, value) in &$id {
-                fmt::Display::fmt(key, $f)?;
+            for (key, value) in $id.iter() {
+                Display::fmt(key, $f)?;
                 $f.write_str(" : ")?;
-                fmt::Display::fmt(value, $f)?;
+                Display::fmt(value, $f)?;
 
                 if count != 1 {
                     $f.write_str(", ")?;
@@ -524,7 +610,7 @@ macro_rules! array_tuple_display {
             for i in 0..5 {
                 // SAFETY: We are in-bounds
                 let it = unsafe { $id.get_unchecked(i) };
-                fmt::Display::fmt(it, $f)?;
+                Display::fmt(it, $f)?;
                 $f.write_str(", ")?;
             }
 
@@ -533,16 +619,16 @@ macro_rules! array_tuple_display {
             for i in len - 5..len - 1 {
                 // SAFETY: We are in-bounds
                 let it = unsafe { $id.get_unchecked(i) };
-                fmt::Display::fmt(it, $f)?;
+                Display::fmt(it, $f)?;
                 $f.write_str(", ")?;
             }
 
             // SAFETY: We are in-bounds
             let it = unsafe { $id.get_unchecked(len) };
-            fmt::Display::fmt(it, $f)?;
+            Display::fmt(it, $f)?;
         } else {
-            for it in &$id {
-                fmt::Display::fmt(it, $f)?;
+            for it in $id.iter() {
+                Display::fmt(it, $f)?;
 
                 if len != 1 {
                     $f.write_str(", ")?;
@@ -554,111 +640,27 @@ macro_rules! array_tuple_display {
     };
 }
 
-impl fmt::Display for Janet {
+impl Display for Janet {
     #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self.unwrap() {
-            TaggedJanet::Boolean(b) => fmt::Display::fmt(&b, f),
-            TaggedJanet::Buffer(s) => fmt::Display::fmt(&s, f),
-            TaggedJanet::Number(n) => fmt::Display::fmt(&n, f),
-            TaggedJanet::String(s) => fmt::Display::fmt(&s, f),
-            TaggedJanet::Abstract(a) => {
-                // Special cases for integers
-                if let Ok(int) = a.get::<i64>() {
-                    return fmt::Display::fmt(int, f);
-                }
-
-                if let Ok(int) = a.get::<u64>() {
-                    return fmt::Display::fmt(int, f);
-                }
-
-                // If the abstract has tostring function
-                if let Some(tostring) = a.type_info().tostring {
-                    let buff = JanetBuffer::with_capacity(30);
-                    unsafe { tostring(a.raw, buff.raw) }
-                    return fmt::Display::fmt(&buff, f);
-                }
-
-                // In case there is no tostring function, default to this
-                f.write_str("<Abstract>")
-            },
-            TaggedJanet::Array(arr) => {
-                f.write_str("@[")?;
-                array_tuple_display!(arr, f);
-                f.write_char(']')
-            },
-            TaggedJanet::CFunction(_) => f.write_str("<C function>"),
-            TaggedJanet::Fiber(fiber) => {
-                f.write_str("<Fiber ")?;
-                fmt::Debug::fmt(&fiber.as_raw(), f)?;
-                f.write_char('>')
-            },
-            TaggedJanet::Function(_) => f.write_str("<Function>"),
-            TaggedJanet::Keyword(key) => fmt::Display::fmt(&key, f),
-            TaggedJanet::Nil => f.write_str("nil"),
-            TaggedJanet::Pointer(ptr) => {
-                f.write_char('<')?;
-                fmt::Pointer::fmt(&ptr, f)?;
-                f.write_char('>')
-            },
-            TaggedJanet::Struct(st) => {
-                f.write_char('{')?;
-                struct_table_display!(st, f);
-                f.write_char('}')
-            },
-            TaggedJanet::Symbol(sym) => fmt::Display::fmt(&sym, f),
-            TaggedJanet::Table(table) => {
-                // Classes in Janet are just tables with prototype with special keywords
-                // If a vailable we printo the class name
-                if let Some(proto) = table.prototype() {
-                    if let Some(name) = proto.get(JanetKeyword::new("_name")) {
-                        match name.unwrap() {
-                            TaggedJanet::Buffer(s) => {
-                                f.write_char('@')?;
-                                fmt::Display::fmt(&s, f)?
-                            },
-                            TaggedJanet::String(s) => {
-                                f.write_char('@')?;
-                                fmt::Display::fmt(&s, f)?
-                            },
-                            TaggedJanet::Symbol(s) => {
-                                f.write_char('@')?;
-                                fmt::Display::fmt(&s, f)?
-                            },
-                            _ => f.write_char('@')?,
-                        }
-                    } else {
-                        f.write_char('@')?;
-                    }
-                } else {
-                    f.write_char('@')?;
-                }
-                f.write_char('{')?;
-                struct_table_display!(table, f);
-                f.write_char('}')
-            },
-            TaggedJanet::Tuple(tup) => {
-                f.write_char('[')?;
-                array_tuple_display!(tup, f);
-                f.write_char(']')
-            },
-        }
+        Display::fmt(&self.unwrap(), f)
     }
 }
 
 #[cfg(feature = "std")]
+#[cfg_attr(feature = "_doc", doc(cfg(feature = "std")))]
 impl error::Error for Janet {}
 
-impl PartialEq<&Janet> for Janet {
+impl PartialEq<&Self> for Janet {
     #[inline]
-    fn eq(&self, other: &&Janet) -> bool {
+    fn eq(&self, other: &&Self) -> bool {
         self.eq(*other)
     }
 }
 
-impl PartialOrd<&Janet> for Janet {
+impl PartialOrd<&Self> for Janet {
     #[inline]
-    fn partial_cmp(&self, other: &&Janet) -> Option<Ordering> {
+    fn partial_cmp(&self, other: &&Self) -> Option<Ordering> {
         self.partial_cmp(*other)
     }
 }
@@ -680,20 +682,17 @@ impl PartialOrd<Janet> for &Janet {
 impl DeepEq for Janet {
     #[inline]
     fn deep_eq(&self, other: &Self) -> bool {
-        // HACK: The left side of the match binding doesn't require `ref` since Rust 1.49
-        // `binding by-move and by-ref in the same pattern is unstable`
-        // If bump the minimal rust version to >= 1.49, remove the first ref of the patterns
         match (self.unwrap(), other.unwrap()) {
-            (TaggedJanet::Array(ref s), TaggedJanet::Array(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Array(ref s), TaggedJanet::Tuple(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Tuple(ref s), TaggedJanet::Array(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Buffer(ref s), TaggedJanet::Buffer(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Buffer(ref s), TaggedJanet::String(ref o)) => s.deep_eq(o),
-            (TaggedJanet::String(ref s), TaggedJanet::Buffer(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Struct(ref s), TaggedJanet::Struct(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Struct(ref s), TaggedJanet::Table(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Table(ref s), TaggedJanet::Struct(ref o)) => s.deep_eq(o),
-            (TaggedJanet::Table(ref s), TaggedJanet::Table(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Array(s), TaggedJanet::Array(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Array(s), TaggedJanet::Tuple(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Tuple(s), TaggedJanet::Array(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Buffer(s), TaggedJanet::Buffer(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Buffer(s), TaggedJanet::String(ref o)) => s.deep_eq(o),
+            (TaggedJanet::String(s), TaggedJanet::Buffer(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Struct(s), TaggedJanet::Struct(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Struct(s), TaggedJanet::Table(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Table(s), TaggedJanet::Struct(ref o)) => s.deep_eq(o),
+            (TaggedJanet::Table(s), TaggedJanet::Table(ref o)) => s.deep_eq(o),
             _ => self.eq(other),
         }
     }
@@ -713,9 +712,9 @@ impl From<&CJanet> for Janet {
     }
 }
 
-impl From<&Janet> for Janet {
+impl From<&Self> for Janet {
     #[inline]
-    fn from(val: &Janet) -> Self {
+    fn from(val: &Self) -> Self {
         *val
     }
 }
@@ -839,6 +838,7 @@ impl From<&char> for Janet {
 }
 
 impl From<JanetCFunction> for Janet {
+    #[inline]
     fn from(val: JanetCFunction) -> Self {
         Self::c_function(val)
     }
@@ -961,6 +961,7 @@ macro_rules! try_from_janet {
 }
 
 from_for_janet!(JanetPointer, pointer);
+from_for_janet!(deref & JanetPointer, pointer);
 try_from_janet!(JanetPointer, TaggedJanet::Pointer);
 
 from_for_janet!(JanetAbstract, j_abstract);
@@ -999,9 +1000,11 @@ from_for_janet!(clone &JanetKeyword<'_>, keyword);
 try_from_janet!(JanetKeyword<'_>, TaggedJanet::Keyword);
 
 from_for_janet!(JanetFunction<'_>, function);
+from_for_janet!(clone &JanetFunction<'_>, function);
 try_from_janet!(JanetFunction<'_>, TaggedJanet::Function);
 
 from_for_janet!(JanetFiber<'_>, fiber);
+from_for_janet!(clone &JanetFiber<'_>, fiber);
 try_from_janet!(JanetFiber<'_>, TaggedJanet::Fiber);
 
 try_from_janet!(JanetCFunction, TaggedJanet::CFunction);
@@ -1097,6 +1100,98 @@ pub enum TaggedJanet<'data> {
     Tuple(JanetTuple<'data>),
 }
 
+impl Display for TaggedJanet<'_> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            TaggedJanet::Boolean(b) => Display::fmt(b, f),
+            TaggedJanet::Buffer(s) => Display::fmt(s, f),
+            TaggedJanet::Number(n) => Display::fmt(n, f),
+            TaggedJanet::String(s) => Display::fmt(s, f),
+            TaggedJanet::Abstract(a) => {
+                // Special cases for integers
+                if let Ok(int) = a.get::<i64>() {
+                    return Display::fmt(int, f);
+                }
+
+                if let Ok(int) = a.get::<u64>() {
+                    return Display::fmt(int, f);
+                }
+
+                // If the abstract has tostring function
+                if let Some(tostring) = a.type_info().tostring {
+                    let buff = JanetBuffer::with_capacity(30);
+                    unsafe { tostring(a.raw, buff.raw) }
+                    return Display::fmt(&buff, f);
+                }
+
+                // In case there is no tostring function, default to this
+                f.write_str("<Abstract>")
+            },
+            TaggedJanet::Array(arr) => {
+                f.write_str("@[")?;
+                array_tuple_display!(arr, f);
+                f.write_char(']')
+            },
+            TaggedJanet::CFunction(_) => f.write_str("<C function>"),
+            TaggedJanet::Fiber(fiber) => {
+                f.write_str("<Fiber ")?;
+                fmt::Debug::fmt(&fiber.as_raw(), f)?;
+                f.write_char('>')
+            },
+            TaggedJanet::Function(_) => f.write_str("<Function>"),
+            TaggedJanet::Keyword(key) => Display::fmt(key, f),
+            TaggedJanet::Nil => f.write_str("nil"),
+            TaggedJanet::Pointer(ptr) => {
+                f.write_char('<')?;
+                fmt::Pointer::fmt(&ptr, f)?;
+                f.write_char('>')
+            },
+            TaggedJanet::Struct(st) => {
+                f.write_char('{')?;
+                struct_table_display!(st, f);
+                f.write_char('}')
+            },
+            TaggedJanet::Symbol(sym) => Display::fmt(sym, f),
+            TaggedJanet::Table(table) => {
+                // Classes in Janet are just tables with prototype with special keywords
+                // If a vailable we printo the class name
+                if let Some(proto) = table.prototype() {
+                    if let Some(name) = proto.get(JanetKeyword::new("_name")) {
+                        match name.unwrap() {
+                            TaggedJanet::Buffer(s) => {
+                                f.write_char('@')?;
+                                Display::fmt(&s, f)?;
+                            },
+                            TaggedJanet::String(s) => {
+                                f.write_char('@')?;
+                                Display::fmt(&s, f)?;
+                            },
+                            TaggedJanet::Symbol(s) => {
+                                f.write_char('@')?;
+                                Display::fmt(&s, f)?;
+                            },
+                            _ => f.write_char('@')?,
+                        }
+                    } else {
+                        f.write_char('@')?;
+                    }
+                } else {
+                    f.write_char('@')?;
+                }
+                f.write_char('{')?;
+                struct_table_display!(table, f);
+                f.write_char('}')
+            },
+            TaggedJanet::Tuple(tup) => {
+                f.write_char('[')?;
+                array_tuple_display!(tup, f);
+                f.write_char(']')
+            },
+        }
+    }
+}
+
 impl From<Janet> for TaggedJanet<'_> {
     #[inline]
     fn from(val: Janet) -> Self {
@@ -1297,7 +1392,7 @@ impl From<u32> for JanetSignal {
 
 impl From<JanetSignal> for u32 {
     fn from(val: JanetSignal) -> Self {
-        val as u32
+        val as _
     }
 }
 
@@ -1442,6 +1537,27 @@ macro_rules! string_impl_partial_eq {
             }
         }
     };
+    (#[cfg($attr:meta)]; $lhs:ty, $rhs:ty) => {
+        #[cfg($attr)]
+        #[cfg_attr(feature = "_doc", doc(cfg($attr)))]
+        impl<'a, 'b> PartialEq<$rhs> for $lhs {
+            #[inline]
+            fn eq(&self, other: &$rhs) -> bool {
+                let other: &[u8] = other.as_ref();
+                PartialEq::eq(self.as_bytes(), other)
+            }
+        }
+
+        #[cfg($attr)]
+        #[cfg_attr(feature = "_doc", doc(cfg($attr)))]
+        impl<'a, 'b> PartialEq<$lhs> for $rhs {
+            #[inline]
+            fn eq(&self, other: &$lhs) -> bool {
+                let this: &[u8] = self.as_ref();
+                PartialEq::eq(this, other.as_bytes())
+            }
+        }
+    };
 }
 
 // Macro to impl PartialOrd for Janet String-like types against Rust String-like types.
@@ -1467,6 +1583,27 @@ macro_rules! string_impl_partial_ord {
             }
         }
     };
+    (#[cfg($attr:meta)]; $lhs:ty, $rhs:ty) => {
+        #[cfg($attr)]
+        #[cfg_attr(feature = "_doc", doc(cfg($attr)))]
+        impl<'a, 'b> PartialOrd<$rhs> for $lhs {
+            #[inline]
+            fn partial_cmp(&self, other: &$rhs) -> Option<Ordering> {
+                let other: &[u8] = other.as_ref();
+                PartialOrd::partial_cmp(self.as_bytes(), other)
+            }
+        }
+
+        #[cfg($attr)]
+        #[cfg_attr(feature = "_doc", doc(cfg($attr)))]
+        impl<'a, 'b> PartialOrd<$lhs> for $rhs {
+            #[inline]
+            fn partial_cmp(&self, other: &$lhs) -> Option<Ordering> {
+                let this: &[u8] = self.as_ref();
+                PartialOrd::partial_cmp(this, other.as_bytes())
+            }
+        }
+    };
 }
 
 impl_string_like!(JanetString<'_> JanetKeyword<'_> JanetSymbol<'_>);
@@ -1482,10 +1619,8 @@ string_impl_partial_eq!(JanetString<'_>, str);
 string_impl_partial_eq!(JanetString<'_>, &'a str);
 string_impl_partial_eq!(JanetString<'_>, bstr::BStr);
 string_impl_partial_eq!(JanetString<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetString<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetString<'_>, &'a bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetString<'_>, bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetString<'_>, &'a bstr::BString);
 
 string_impl_partial_ord!(JanetString<'_>, Vec<u8>);
 string_impl_partial_ord!(JanetString<'_>, [u8]);
@@ -1495,10 +1630,8 @@ string_impl_partial_ord!(JanetString<'_>, str);
 string_impl_partial_ord!(JanetString<'_>, &'a str);
 string_impl_partial_ord!(JanetString<'_>, bstr::BStr);
 string_impl_partial_ord!(JanetString<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetString<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetString<'_>, &'a bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetString<'_>, bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetString<'_>, &'a bstr::BString);
 
 string_impl_partial_eq!(JanetBuffer<'_>, Vec<u8>);
 string_impl_partial_eq!(JanetBuffer<'_>, [u8]);
@@ -1508,10 +1641,8 @@ string_impl_partial_eq!(JanetBuffer<'_>, str);
 string_impl_partial_eq!(JanetBuffer<'_>, &'a str);
 string_impl_partial_eq!(JanetBuffer<'_>, bstr::BStr);
 string_impl_partial_eq!(JanetBuffer<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetBuffer<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetBuffer<'_>, &'a bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetBuffer<'_>, bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetBuffer<'_>, &'a bstr::BString);
 
 string_impl_partial_ord!(JanetBuffer<'_>, Vec<u8>);
 string_impl_partial_ord!(JanetBuffer<'_>, [u8]);
@@ -1521,10 +1652,8 @@ string_impl_partial_ord!(JanetBuffer<'_>, str);
 string_impl_partial_ord!(JanetBuffer<'_>, &'a str);
 string_impl_partial_ord!(JanetBuffer<'_>, bstr::BStr);
 string_impl_partial_ord!(JanetBuffer<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetBuffer<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetBuffer<'_>, &'a bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetBuffer<'_>, bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetBuffer<'_>, &'a bstr::BString);
 
 string_impl_partial_eq!(JanetSymbol<'_>, Vec<u8>);
 string_impl_partial_eq!(JanetSymbol<'_>, [u8]);
@@ -1534,10 +1663,8 @@ string_impl_partial_eq!(JanetSymbol<'_>, str);
 string_impl_partial_eq!(JanetSymbol<'_>, &'a str);
 string_impl_partial_eq!(JanetSymbol<'_>, bstr::BStr);
 string_impl_partial_eq!(JanetSymbol<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetSymbol<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetSymbol<'_>, &'a bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetSymbol<'_>, bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetSymbol<'_>, &'a bstr::BString);
 
 string_impl_partial_ord!(JanetSymbol<'_>, Vec<u8>);
 string_impl_partial_ord!(JanetSymbol<'_>, [u8]);
@@ -1547,10 +1674,8 @@ string_impl_partial_ord!(JanetSymbol<'_>, str);
 string_impl_partial_ord!(JanetSymbol<'_>, &'a str);
 string_impl_partial_ord!(JanetSymbol<'_>, bstr::BStr);
 string_impl_partial_ord!(JanetSymbol<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetSymbol<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetSymbol<'_>, &'a bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetSymbol<'_>, bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetSymbol<'_>, &'a bstr::BString);
 
 string_impl_partial_eq!(JanetKeyword<'_>, Vec<u8>);
 string_impl_partial_eq!(JanetKeyword<'_>, [u8]);
@@ -1560,10 +1685,8 @@ string_impl_partial_eq!(JanetKeyword<'_>, str);
 string_impl_partial_eq!(JanetKeyword<'_>, &'a str);
 string_impl_partial_eq!(JanetKeyword<'_>, bstr::BStr);
 string_impl_partial_eq!(JanetKeyword<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetKeyword<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_eq!(JanetKeyword<'_>, &'a bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetKeyword<'_>, bstr::BString);
+string_impl_partial_eq!(#[cfg(feature = "std")]; JanetKeyword<'_>, &'a bstr::BString);
 
 string_impl_partial_ord!(JanetKeyword<'_>, Vec<u8>);
 string_impl_partial_ord!(JanetKeyword<'_>, [u8]);
@@ -1573,10 +1696,8 @@ string_impl_partial_ord!(JanetKeyword<'_>, str);
 string_impl_partial_ord!(JanetKeyword<'_>, &'a str);
 string_impl_partial_ord!(JanetKeyword<'_>, bstr::BStr);
 string_impl_partial_ord!(JanetKeyword<'_>, &'a bstr::BStr);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetKeyword<'_>, bstr::BString);
-#[cfg(feature = "std")]
-string_impl_partial_ord!(JanetKeyword<'_>, &'a bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetKeyword<'_>, bstr::BString);
+string_impl_partial_ord!(#[cfg(feature = "std")]; JanetKeyword<'_>, &'a bstr::BString);
 
 #[cfg(all(test, any(feature = "amalgation", feature = "link-system")))]
 mod tests {
