@@ -423,7 +423,7 @@ impl<'data> JanetTable<'data> {
     /// Returns a reference to the value corresponding to the `key` without checking for
     /// anything.
     ///
-    /// # SAFETY
+    /// # Safety
     /// This function doesn't check for null pointer and if the key or value as Janet nil
     #[inline]
     pub(crate) unsafe fn get_unchecked(&self, key: impl Into<Janet>) -> &'data Janet {
@@ -433,7 +433,7 @@ impl<'data> JanetTable<'data> {
     /// Returns a mutable reference to the value corresponding to the `key` without
     /// checking for anything.
     ///
-    /// # SAFETY
+    /// # Safety
     /// This function doesn't check for null pointer and if the key or value as Janet nil
     #[inline]
     pub(crate) unsafe fn get_mut_unchecked(&mut self, key: impl Into<Janet>) -> &'data mut Janet {
@@ -443,7 +443,7 @@ impl<'data> JanetTable<'data> {
     /// Returns the key-value pair corresponding to the supplied `key`, with a mutable
     /// reference to value without checking for anything.
     ///
-    /// # SAFETY
+    /// # Safety
     /// This function doesn't check for null pointer and if the key or value as Janet nil
     #[inline]
     pub(crate) unsafe fn get_key_value_mut_unchecked(
@@ -466,7 +466,7 @@ impl<'data> JanetTable<'data> {
     /// Returns the key-value pair corresponding to the supplied `key`, with a reference
     /// to value without checking for anything.
     ///
-    /// # SAFETY
+    /// # Safety
     /// This function doesn't check for null pointer and if the key or value as Janet nil
     #[inline]
     pub(crate) unsafe fn get_key_value_unchecked(
@@ -720,7 +720,7 @@ impl<'data> JanetTable<'data> {
     /// assert_eq!(table.get_owned(10), Some(Janet::from("ten")));
     /// assert_eq!(table.get_owned(11), None);
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn get_owned(&self, key: impl Into<Janet>) -> Option<Janet> {
         let key = key.into();
 
@@ -749,7 +749,7 @@ impl<'data> JanetTable<'data> {
     /// );
     /// assert_eq!(table.get_owned_key_value(11), None);
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn get_owned_key_value(&self, key: impl Into<Janet>) -> Option<(Janet, Janet)> {
         let key = key.into();
         self.get_owned(key).map(|v| (key, v))
@@ -769,7 +769,7 @@ impl<'data> JanetTable<'data> {
     /// assert_eq!(table.raw_get_owned(10), Some(Janet::from("ten")));
     /// assert_eq!(table.raw_get_owned(11), None);
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn raw_get_owned(&self, key: impl Into<Janet>) -> Option<Janet> {
         let key = key.into();
 
@@ -798,7 +798,7 @@ impl<'data> JanetTable<'data> {
     /// );
     /// assert_eq!(table.raw_get_owned_key_value(11), None);
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn raw_get_owned_key_value(&self, key: impl Into<Janet>) -> Option<(Janet, Janet)> {
         let key = key.into();
         self.raw_get_owned(key).map(|v| (key, v))
@@ -957,6 +957,43 @@ impl<'data> JanetTable<'data> {
         old_value
     }
 
+    /// Tries to insert a key-value pair into the map, and returns
+    /// a mutable reference to the value in the entry.
+    ///
+    /// # Errors
+    ///
+    /// If the map already had this key present, nothing is updated, and
+    /// an error containing the occupied entry and the value is returned.
+    ///
+    /// # Examples
+    ///
+    /// Basic usage:
+    ///
+    /// ```
+    /// use janetrs::{Janet, JanetTable};
+    /// # let _client = janetrs::client::JanetClient::init().unwrap();
+    ///
+    /// let mut map = JanetTable::new();
+    /// assert_eq!(map.try_insert(37, "a").unwrap(), &Janet::from("a"));
+    ///
+    /// let err = map.try_insert(37, "b").unwrap_err();
+    /// assert_eq!(err.entry.key(), &Janet::from(37));
+    /// assert_eq!(err.entry.get(), &Janet::from("a"));
+    /// assert_eq!(err.value, Janet::from("b"));
+    /// ```
+    #[cfg_attr(feature = "inline-more", inline)]
+    pub fn try_insert(
+        &mut self, key: impl Into<Janet>, value: impl Into<Janet>,
+    ) -> Result<&mut Janet, OccupiedError<'_, 'data>> {
+        match self.entry(key) {
+            Entry::Occupied(entry) => Err(OccupiedError {
+                entry,
+                value: value.into(),
+            }),
+            Entry::Vacant(entry) => Ok(entry.insert(value)),
+        }
+    }
+
     /// Returns `true` if the table contains a value for the specified `key`.
     ///
     /// # Examples
@@ -970,7 +1007,7 @@ impl<'data> JanetTable<'data> {
     /// assert!(table.contains_key(10));
     /// assert!(!table.contains_key(11));
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn contains_key(&self, key: impl Into<Janet>) -> bool {
         self.get(key).is_some()
     }
@@ -988,7 +1025,7 @@ impl<'data> JanetTable<'data> {
     /// assert!(table.contains("ten"));
     /// assert!(!table.contains(11));
     /// ```
-    #[inline]
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn contains(&self, value: impl Into<Janet>) -> bool {
         let value = value.into();
         self.values().any(|&v| v == value)
@@ -1519,7 +1556,7 @@ pub struct OccupiedEntry<'a, 'data> {
     table: &'a mut JanetTable<'data>,
 }
 
-impl<'a> OccupiedEntry<'a, '_> {
+impl<'a, 'data> OccupiedEntry<'a, 'data> {
     /// Gets a reference to the value in the entry.
     ///
     /// # Examples
@@ -1802,6 +1839,45 @@ impl<'a, 'data> VacantEntry<'a, 'data> {
     }
 }
 
+
+/// The error returned by [`try_insert`](JanetTable::try_insert) when the key already
+/// exists.
+///
+/// Contains the occupied entry, and the value that was not inserted.
+pub struct OccupiedError<'a, 'data> {
+    /// The entry in the map that was already occupied.
+    pub entry: OccupiedEntry<'a, 'data>,
+    /// The value which was not inserted, because the entry was already occupied.
+    pub value: Janet,
+}
+
+impl Debug for OccupiedError<'_, '_> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("OccupiedError")
+            .field("key", self.entry.key())
+            .field("old_value", self.entry.get())
+            .field("new_value", &self.value)
+            .finish()
+    }
+}
+
+impl<'a, 'data> fmt::Display for OccupiedError<'a, 'data> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_fmt(format_args!(
+            "failed to insert {:?}, key {:?} already exists with value {:?}",
+            self.value,
+            self.entry.key(),
+            self.entry.get()
+        ))
+    }
+}
+
+#[cfg(feature = "std")]
+#[cfg_attr(feature = "_doc", doc(cfg(feature = "std")))]
+impl std::error::Error for OccupiedError<'_, '_> {}
+
 /// An iterator over a reference to the [`JanetTable`] key-value pairs.
 #[derive(Clone)]
 pub struct Iter<'a, 'data> {
