@@ -127,7 +127,7 @@ impl<'data> JanetTuple<'data> {
 
     // Get the [`JanetTupleHead`] from the `JanetStruct` pointer.
     fn head(&self) -> &JanetTupleHead {
-        // Safety: Janet tuple are always be a valid pointer
+        // SAFETY: Janet tuple are always be a valid pointer
         unsafe { &*janet_tuple_head(self.raw) }
     }
 
@@ -211,6 +211,7 @@ impl<'data> JanetTuple<'data> {
     /// let tup = tuple![1.0, "foo", 4.0];
     /// assert!(tup.contains("foo"));
     /// ```
+    #[cfg_attr(feature = "inline-more", inline)]
     pub fn contains(&self, value: impl Into<Janet>) -> bool {
         let value = value.into();
         self.iter().any(|&elem| elem == value)
@@ -388,6 +389,7 @@ impl<'data> JanetTuple<'data> {
     /// // this will panic at runtime
     /// b"0123456789abcdef".repeat(usize::MAX);
     /// ```
+    #[inline]
     pub fn repeat(&self, n: usize) -> JanetArray {
         self.as_ref().repeat(n).into_iter().collect()
     }
@@ -418,6 +420,7 @@ impl<'data> JanetTuple<'data> {
     /// let v = tuple![];
     /// assert!(v.starts_with(&[]));
     /// ```
+    #[inline]
     pub fn starts_with(&self, needle: &[Janet]) -> bool {
         self.as_ref().starts_with(needle)
     }
@@ -448,6 +451,7 @@ impl<'data> JanetTuple<'data> {
     /// let v = tuple![];
     /// assert!(v.ends_with(&[]));
     /// ```
+    #[inline]
     pub fn ends_with(&self, needle: &[Janet]) -> bool {
         self.as_ref().ends_with(needle)
     }
@@ -499,6 +503,7 @@ impl<'data> JanetTuple<'data> {
     ///     tuple![0, 1, 1, 1, 1, 2, 3, 5, 8, 13, 21, 34, 42, 55].as_ref()
     /// );
     /// ```
+    #[inline]
     pub fn binary_search(&self, x: &Janet) -> Result<usize, usize> {
         self.binary_search_by(|p| p.cmp(x))
     }
@@ -976,7 +981,7 @@ impl Clone for JanetTuple<'_> {
         let len = self.len();
         let mut clone = Self::builder(len);
 
-        for elem in self.into_iter().cloned() {
+        for elem in self.into_iter().copied() {
             clone = clone.put(elem);
         }
 
@@ -985,17 +990,16 @@ impl Clone for JanetTuple<'_> {
 }
 
 impl PartialOrd for JanetTuple<'_> {
+    #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        use core::cmp::Ordering::*;
+        use core::cmp::Ordering::{Equal, Greater, Less};
 
         match self.len().cmp(&other.len()) {
-            x @ Less => Some(x),
-            x @ Greater => Some(x),
+            x @ (Less | Greater) => Some(x),
             Equal => {
                 for (s, o) in self.iter().zip(other.iter()) {
                     match s.partial_cmp(o) {
-                        x @ Some(Less) => return x,
-                        x @ Some(Greater) => return x,
+                        x @ Some(Less | Greater) => return x,
                         Some(Equal) => continue,
                         None => return None,
                     }
@@ -1007,17 +1011,16 @@ impl PartialOrd for JanetTuple<'_> {
 }
 
 impl Ord for JanetTuple<'_> {
+    #[inline]
     fn cmp(&self, other: &Self) -> Ordering {
-        use core::cmp::Ordering::*;
+        use core::cmp::Ordering::{Equal, Greater, Less};
 
         match self.len().cmp(&other.len()) {
-            x @ Less => x,
-            x @ Greater => x,
+            x @ (Less | Greater) => x,
             Equal => {
                 for (s, o) in self.iter().zip(other.iter()) {
                     match s.cmp(o) {
-                        x @ Less => return x,
-                        x @ Greater => return x,
+                        x @ (Less | Greater) => return x,
                         Equal => continue,
                     }
                 }
@@ -1036,7 +1039,7 @@ impl PartialEq for JanetTuple<'_> {
         }
 
         // If the hash is the same
-        // Safety: Janet tuple must always be a valid ponter
+        // SAFETY: Janet tuple must always be a valid ponter
         if self.head().hash.eq(&self.head().hash) {
             return true;
         }
@@ -1069,7 +1072,7 @@ impl Default for JanetTuple<'_> {
 impl AsRef<[Janet]> for JanetTuple<'_> {
     #[inline]
     fn as_ref(&self) -> &[Janet] {
-        // Safety: Janet uses i32 as max size for all collections and indexing, so it always has
+        // SAFETY: Janet uses i32 as max size for all collections and indexing, so it always has
         // len lesser than isize::MAX
         unsafe { core::slice::from_raw_parts(self.raw as *const _, self.len() as usize) }
     }
@@ -1241,7 +1244,7 @@ impl<'a> Iterator for IntoIter<'_> {
         if self.index_head >= self.index_tail {
             None
         } else {
-            let ret = self.tup.get(self.index_head).cloned();
+            let ret = self.tup.get(self.index_head).copied();
             self.index_head += 1;
             ret
         }
@@ -1261,7 +1264,7 @@ impl DoubleEndedIterator for IntoIter<'_> {
             None
         } else {
             self.index_tail -= 1;
-            self.tup.get(self.index_tail).cloned()
+            self.tup.get(self.index_tail).copied()
         }
     }
 }
